@@ -2,7 +2,7 @@
  * dselect - Debian package maintenance user interface
  * baselist.cc - list of somethings
  *
- * Copyright © 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1994,1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2001 Wichert Akkerman <wakkerma@debian.org>
  * Copyright © 2007-2013 Guillem Jover <guillem@debian.org>
  *
@@ -25,7 +25,6 @@
 
 #include <sys/ioctl.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <string.h>
 #include <termios.h>
@@ -49,70 +48,14 @@ void mywerase(WINDOW *win) {
   wmove(win,0,0);
 }
 
-baselist *baselist::signallist = nullptr;
-void baselist::sigwinchhandler(int) {
-  int save_errno = errno;
-  struct winsize size;
-  debug(dbg_general, "baselist::sigwinchhandler(), signallist=%p", signallist);
-  baselist *p= signallist;
-  p->enddisplay();
-  endwin(); initscr();
-  if (ioctl(fileno(stdout), TIOCGWINSZ, &size) != 0) ohshite(_("ioctl(TIOCGWINSZ) failed"));
-  resizeterm(size.ws_row, size.ws_col); wrefresh(curscr);
-  p->startdisplay();
-  if (doupdate() == ERR) ohshite(_("doupdate in SIGWINCH handler failed"));
-  errno = save_errno;
-}
-
-static void cu_sigwinch(int, void **argv) {
-  struct sigaction *osigactp= (struct sigaction*)argv[0];
-  sigset_t *oblockedp= (sigset_t*)argv[1];
-
-  if (sigaction(SIGWINCH, osigactp, nullptr))
-    ohshite(_("failed to restore old SIGWINCH sigact"));
-  delete osigactp;
-  if (sigprocmask(SIG_SETMASK, oblockedp, nullptr))
-    ohshite(_("failed to restore old signal mask"));
-  delete oblockedp;
-}
-
 void
-baselist::sigwinch_mask(int how)
+baselist::resize_window()
 {
-  sigset_t sigwinchset;
-  sigemptyset(&sigwinchset);
-  sigaddset(&sigwinchset,SIGWINCH);
-
-  int rc = sigprocmask(how, &sigwinchset, nullptr);
-  if (rc < 0) {
-    if (how == SIG_UNBLOCK)
-      ohshite(_("failed to unblock SIGWINCH"));
-    else
-      ohshite(_("failed to block SIGWINCH"));
-  }
-}
-
-void
-baselist::setupsigwinch()
-{
-  struct sigaction *osigactp = new(struct sigaction);
-  sigset_t *oblockedp = new(sigset_t);
-  if (sigprocmask(0, nullptr, oblockedp))
-    ohshite(_("failed to get old signal mask"));
-  if (sigaction(SIGWINCH, nullptr, osigactp))
-    ohshite(_("failed to get old SIGWINCH sigact"));
-
-  push_cleanup(cu_sigwinch, ~0, nullptr, 0, 2, osigactp, oblockedp);
-
-  sigwinch_mask(SIG_BLOCK);
-
-  struct sigaction nsigact;
-  memset(&nsigact,0,sizeof(nsigact));
-  nsigact.sa_handler= sigwinchhandler;
-  sigemptyset(&nsigact.sa_mask);
-//nsigact.sa_flags= SA_INTERRUPT;
-  if (sigaction(SIGWINCH, &nsigact, nullptr))
-    ohshite(_("failed to set new SIGWINCH sigact"));
+  debug(dbg_general, "baselist::resize_window(), baselist=%p", this);
+  enddisplay();
+  startdisplay();
+  if (doupdate() == ERR)
+    ohshite(_("cannot update screen after window resize"));
 }
 
 void
@@ -136,26 +79,29 @@ baselist::end_column(column &col, const char *title)
 }
 
 void
-baselist::draw_column_head(column &col)
+baselist::draw_column_head(const column &col)
 {
   mvwaddnstr(colheadspad, 0, col.x, col.title, col.width);
 }
 
 void
-baselist::draw_column_sep(column &col, int y)
+baselist::draw_column_sep(const column &col, int y)
 {
   mvwaddch(listpad, y, col.x - 1, ' ');
 }
 
 void
-baselist::draw_column_item(column &col, int y, const char *item)
+baselist::draw_column_item(const column &col, int y, const char *item)
 {
   mvwprintw(listpad, y, col.x, "%-*.*s", col.width, col.width, item);
 }
 
 void baselist::setheights() {
   int y= ymax - (title_height + colheads_height + thisstate_height);
-  assert(y>=1);
+
+  if (y < 1)
+    internerr("widget y=%d < 1", y);
+
   if (showinfo==2 && y>=7) {
     list_height= 5;
     whatinfo_height= 1;
@@ -200,7 +146,7 @@ void baselist::startdisplay() {
     part_attr[selstatesel] = A_STANDOUT;
     part_attr[colheads]= A_BOLD;
     part_attr[query] = part_attr[title];
-    part_attr[info] = part_attr[list];
+    part_attr[info_body] = part_attr[list];
     part_attr[info_head] = A_BOLD;
     part_attr[whatinfo] = part_attr[thisstate];
     part_attr[helpscreen] = A_NORMAL;
@@ -236,8 +182,8 @@ void baselist::startdisplay() {
 
   infopad= newpad(MAX_DISPLAY_INFO, total_width);
   if (!infopad) ohshite(_("failed to create info pad"));
-  wattrset(infopad, part_attr[info]);
-  wbkgdset(infopad, ' ' | part_attr[info]);
+  wattrset(infopad, part_attr[info_body]);
+  wbkgdset(infopad, ' ' | part_attr[info_body]);
 
   querywin= newwin(1,xmax,ymax-1,0);
   if (!querywin) ohshite(_("failed to create query window"));
@@ -388,7 +334,7 @@ void baselist::refreshinfo() {
     mvwaddstr(whatinfowin,0,0, whatinfovb.string());
     if (infolines > info_height) {
       wprintw(whatinfowin,_("  -- %d%%, press "),
-              (int)((infotopofscreen + info_height) * 100.0 / infolines));
+              (infotopofscreen + info_height) * 100 / infolines);
       if (infotopofscreen + info_height < infolines) {
         wprintw(whatinfowin,_("%s for more"), bindings->find("iscrollon"));
         if (infotopofscreen) waddstr(whatinfowin, ", ");
@@ -402,14 +348,15 @@ void baselist::refreshinfo() {
 }
 
 void baselist::wordwrapinfo(int offset, const char *m) {
-  int usemax= xmax-5;
+  ssize_t usemax = xmax - 5;
   debug(dbg_general, "baselist[%p]::wordwrapinfo(%d, '%s')", this, offset, m);
   bool wrapping = false;
 
   for (;;) {
     int offleft=offset; while (*m == ' ' && offleft>0) { m++; offleft--; }
-    const char *p= strchr(m,'\n');
-    int l= p ? (int)(p-m) : strlen(m);
+    const char *p = strchrnul(m, '\n');
+    ptrdiff_t l = p - m;
+
     while (l && c_isspace(m[l - 1]))
       l--;
     if (!l || (*m == '.' && l == 1)) {
@@ -434,11 +381,11 @@ void baselist::wordwrapinfo(int offset, const char *m) {
       }
       for (;;) {
         getyx(infopad, y,x);
-        int dosend= usemax-x;
+        ssize_t dosend = usemax - x;
         if (l <= dosend) {
           dosend=l;
         } else {
-          int i=dosend;
+          ssize_t i = dosend;
           while (i > 0 && m[i] != ' ') i--;
           if (i > 0 || x > 0) dosend=i;
         }
@@ -450,7 +397,8 @@ void baselist::wordwrapinfo(int offset, const char *m) {
       }
       wrapping = true;
     }
-    if (!p) break;
+    if (*p == '\0')
+      break;
     if (getcury(infopad) == (MAX_DISPLAY_INFO - 1)) {
       waddstr(infopad,
               "[The package description is too long and has been truncated...]");
