@@ -186,6 +186,38 @@ invalid:
 	return &tki_unknown;
 }
 
+static const char *
+trig_dump_trigger_options(enum trig_options opts)
+{
+	if (opts == TRIG_NOAWAIT)
+		return "/noawait";
+	return "";
+}
+
+/*
+ * Modifies the trigger string by removing the options at the ‘/’ delimiter
+ * and truncating it with a NUL character.
+ */
+static enum trig_options
+trig_parse_trigger_options(char *trigger)
+{
+	char *slash;
+
+	slash = strchr(trigger, '/');
+	if (slash == NULL)
+		return TRIG_AWAIT;
+
+	*slash++ = '\0';
+	if (strcmp("noawait", slash) == 0)
+		return TRIG_NOAWAIT;
+	if (strcmp("await", slash) == 0)
+		return TRIG_AWAIT;
+
+	/* XXX: Should perhaps warn or error for unknown keywords. */
+
+	return TRIG_AWAIT;
+}
+
 /*
  * Calling sequence is:
  *  trig_activate_start(triggername);
@@ -256,8 +288,7 @@ trk_explicit_start(const char *trig)
 	trk_explicit_activate_done();
 
 	varbuf_reset(&trk_explicit_fn);
-	varbuf_add_str(&trk_explicit_fn, triggersdir);
-	varbuf_add_char(&trk_explicit_fn, '/');
+	varbuf_add_dir(&trk_explicit_fn, triggersdir);
 	varbuf_add_str(&trk_explicit_fn, trig);
 	varbuf_end_str(&trk_explicit_fn);
 
@@ -297,17 +328,9 @@ trk_explicit_activate_awaiter(struct pkginfo *aw)
 
 	while (trk_explicit_fgets(buf, sizeof(buf)) >= 0) {
 		struct dpkg_error err;
-		char *slash;
-		bool noawait = false;
-		slash = strchr(buf, '/');
-		if (slash && strcmp("/noawait", slash) == 0) {
-			noawait = true;
-			*slash = '\0';
-		}
-		if (slash && strcmp("/await", slash) == 0) {
-			noawait = false;
-			*slash = '\0';
-		}
+		enum trig_options opts;
+
+		opts = trig_parse_trigger_options(buf);
 
 		pend = pkg_spec_parse_pkg(buf, &err);
 		if (pend == NULL)
@@ -315,7 +338,7 @@ trk_explicit_activate_awaiter(struct pkginfo *aw)
 			         "illegal package name '%.250s': %.250s"),
 			       trk_explicit_fn.buf, buf, err.str);
 
-		trig_record_activation(pend, noawait ? NULL : aw,
+		trig_record_activation(pend, opts == TRIG_NOAWAIT ? NULL : aw,
 		                       trk_explicit_trig);
 	}
 }
@@ -334,19 +357,31 @@ trk_explicit_interest_change(const char *trig,  struct pkginfo *pkg,
 	atomic_file_open(file);
 
 	while (trk_explicit_f && trk_explicit_fgets(buf, sizeof(buf)) >= 0) {
-		const char *pkgname = pkgbin_name(pkg, pkgbin, pnaw_nonambig);
-		size_t len = strlen(pkgname);
+		enum trig_options trig_opts;
+		struct pkginfo *pkg_parsed;
+		struct dpkg_error err;
 
-		if (strncmp(buf, pkgname, len) == 0 && len < sizeof(buf) &&
-		    (buf[len] == '\0' || buf[len] == '/'))
+		trig_opts = trig_parse_trigger_options(buf);
+
+		pkg_parsed = pkg_spec_parse_pkg(buf, &err);
+		if (pkg_parsed == NULL)
+			ohshit(_("trigger interest file '%.250s' syntax error; "
+			         "illegal package name '%.250s': %.250s"),
+			       trk_explicit_fn.buf, buf, err.str);
+
+		if (pkg == pkg_parsed &&
+		    (pkgbin == &pkg_parsed->installed ||
+		     pkgbin == &pkg_parsed->available))
 			continue;
-		fprintf(file->fp, "%s\n", buf);
+
+		fprintf(file->fp, "%s%s\n", buf,
+		        trig_dump_trigger_options(trig_opts));
 		empty = false;
 	}
 	if (signum > 0) {
 		fprintf(file->fp, "%s%s\n",
-		        pkgbin_name(pkg, pkgbin, pnaw_nonambig),
-		        (opts == TRIG_NOAWAIT) ? "/noawait" : "");
+		        pkgbin_name(pkg, pkgbin, pnaw_same),
+		        trig_dump_trigger_options(opts));
 		empty = false;
 	}
 
@@ -375,6 +410,8 @@ static const struct trigkindinfo tki_explicit = {
 /*---------- File triggers. ----------*/
 
 static struct {
+	/* cppcheck-suppress[unusedStructMember]:
+	 * False positive, macros from dlist.h use the tail member. */
 	struct trigfileint *head, *tail;
 } filetriggers;
 
@@ -463,8 +500,8 @@ trig_file_interests_update(void)
 
 	for (tfi = filetriggers.head; tfi; tfi = tfi->inoverall.next)
 		fprintf(file->fp, "%s %s%s\n", trigh.namenode_name(tfi->fnn),
-		        pkgbin_name(tfi->pkg, tfi->pkgbin, pnaw_nonambig),
-		        (tfi->options == TRIG_NOAWAIT) ? "/noawait" : "");
+		        pkgbin_name(tfi->pkg, tfi->pkgbin, pnaw_same),
+		        trig_dump_trigger_options(tfi->options));
 
 	atomic_file_sync(file);
 	atomic_file_close(file);
@@ -510,23 +547,15 @@ trig_file_interests_ensure(void)
 	push_cleanup(cu_closestream, ~0, 1, f);
 	while (fgets_checked(linebuf, sizeof(linebuf), f, triggersfilefile) >= 0) {
 		struct dpkg_error err;
-		char *slash;
-		enum trig_options trig_opts = TRIG_AWAIT;
+		enum trig_options trig_opts;
+
 		space = strchr(linebuf, ' ');
 		if (!space || linebuf[0] != '/')
 			ohshit(_("syntax error in file triggers file '%.250s'"),
 			       triggersfilefile);
 		*space++ = '\0';
 
-		slash = strchr(space, '/');
-		if (slash && strcmp("/noawait", slash) == 0) {
-			trig_opts = TRIG_NOAWAIT;
-			*slash = '\0';
-		}
-		if (slash && strcmp("/await", slash) == 0) {
-			trig_opts = TRIG_AWAIT;
-			*slash = '\0';
-		}
+		trig_opts = trig_parse_trigger_options(space);
 
 		pkg = pkg_spec_parse_pkg(space, &err);
 		if (pkg == NULL)
