@@ -1,7 +1,7 @@
 # Copyright © 1996 Ian Jackson
 # Copyright © 2005 Frank Lichtenheld <frank@lichtenheld.de>
 # Copyright © 2009 Raphaël Hertzog <hertzog@debian.org>
-# Copyright © 2012-2015 Guillem Jover <guillem@debian.org>
+# Copyright © 2012-2017 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,11 +24,17 @@ Dpkg::Changelog::Debian - parse Debian changelogs
 
 =head1 DESCRIPTION
 
+This class represents a Debian changelog file as an array of changelog
+entries (Dpkg::Changelog::Entry::Debian).
+It implements the generic interface Dpkg::Changelog.
+Only methods specific to this implementation are described below,
+the rest are inherited.
+
 Dpkg::Changelog::Debian parses Debian changelogs as described in
 deb-changelog(5).
 
 The parser tries to ignore most cruft like # or /* */ style comments,
-CVS comments, vim variables, emacs local variables and stuff from
+RCS keywords, Vim modelines, Emacs local variables and stuff from
 older changelogs with other formats at the end of the file.
 NOTE: most of these are ignored silently currently, there is no
 parser error issued for them. This should become configurable in the
@@ -118,10 +124,11 @@ my $ancient_delimiter_re = qr{
 
 =over 4
 
-=item $c->parse($fh, $description)
+=item $count = $c->parse($fh, $description)
 
-Read the filehandle and parse a Debian changelog in it. The data in the
-object is reset before parsing new data.
+Read the filehandle and parse a Debian changelog in it, to store the entries
+as an array of Dpkg::Changelog::Entry::Debian objects.
+Any previous entries in the object are reset before parsing new data.
 
 Returns the number of changelog entries that have been parsed with success.
 
@@ -139,7 +146,8 @@ sub parse {
     my $expect = FIRST_HEADING;
     my $entry = Dpkg::Changelog::Entry::Debian->new();
     my @blanklines = ();
-    my $unknowncounter = 1; # to make version unique, e.g. for using as id
+    # To make version unique, for example for using as id.
+    my $unknowncounter = 1;
     local $_;
 
     while (<$fh>) {
@@ -159,14 +167,18 @@ sub parse {
 	    foreach my $error ($entry->parse_header()) {
 		$self->parse_error($file, $., $error, $_);
 	    }
-	    $expect= START_CHANGES;
+            $expect = START_CHANGES;
 	    @blanklines = ();
 	} elsif (m/^(?:;;\s*)?Local variables:/io) {
-	    last; # skip Emacs variables at end of file
+            # Save any trailing Emacs variables at end of file.
+            $self->set_unparsed_tail("$_\n" . (file_slurp($fh) // ''));
+            last;
 	} elsif (m/^vim:/io) {
-	    last; # skip vim variables at end of file
+            # Save any trailing Vim modelines at end of file.
+            $self->set_unparsed_tail("$_\n" . (file_slurp($fh) // ''));
+            last;
 	} elsif (m/^\$\w+:.*\$/o) {
-	    next; # skip stuff that look like a CVS keyword
+	    next; # skip stuff that look like a RCS keyword
 	} elsif (m/^\# /o) {
 	    next; # skip comments, even that's not supported
 	} elsif (m{^/\*.*\*/}o) {
@@ -243,7 +255,6 @@ sub parse {
 }
 
 1;
-__END__
 
 =back
 
