@@ -20,7 +20,7 @@ package Dpkg::Checksums;
 use strict;
 use warnings;
 
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 our @EXPORT = qw(
     checksums_is_supported
     checksums_get_list
@@ -41,7 +41,7 @@ Dpkg::Checksums - generate and manipulate file checksums
 
 =head1 DESCRIPTION
 
-This module provides an object that can generate and manipulate
+This module provides a class that can generate and manipulate
 various file checksums as well as some methods to query information
 about supported checksums.
 
@@ -106,10 +106,6 @@ whether the checksum algorithm is considered cryptographically strong.
 sub checksums_get_property($$) {
     my ($alg, $property) = @_;
 
-    if ($property eq 'program') {
-        warnings::warnif('deprecated', 'obsolete checksums program property');
-    }
-
     return unless checksums_is_supported($alg);
     return $CHECKSUMS->{lc($alg)}{$property};
 }
@@ -155,9 +151,9 @@ sub reset {
 
 =item $ck->add_from_file($filename, %opts)
 
-Add checksums information for the file $filename. The file must exists
-for the call to succeed. If you don't want the given filename to appear
-when you later export the checksums you might want to set the "key"
+Add or verify checksums information for the file $filename. The file must
+exists for the call to succeed. If you don't want the given filename to
+appear when you later export the checksums you might want to set the "key"
 option with the public name that you want to use. Also if you don't want
 to generate all the checksums, you can pass an array reference of the
 wanted checksums in the "checksums" option.
@@ -232,7 +228,7 @@ sub add_from_string {
 		  $alg, $checksum);
 	}
 	my ($sum, $size, $file) = ($1, $2, $3);
-	if (not $opts{update} and  exists($checksums->{$file}{$alg})
+	if (not $opts{update} and exists($checksums->{$file}{$alg})
 	    and $checksums->{$file}{$alg} ne $sum) {
 	    error(g_("conflicting checksums '%s' and '%s' for file '%s'"),
 		  $checksums->{$file}{$alg}, $sum, $file);
@@ -400,6 +396,10 @@ sub export_to_control {
 
 =head1 CHANGES
 
+=head2 Version 1.04 (dpkg 1.20.0)
+
+Remove warning: For obsolete property 'program'.
+
 =head2 Version 1.03 (dpkg 1.18.5)
 
 New property: Add new 'strong' property.
