@@ -30,7 +30,7 @@ use parent qw(Dpkg::Control::HashCore);
 
 =head1 NAME
 
-Dpkg::Control::Hash - parse and manipulate a block of RFC822-like fields
+Dpkg::Control::Hash - parse and manipulate a stanza of deb822 fields
 
 =head1 DESCRIPTION
 
@@ -39,14 +39,10 @@ field knowledge.
 
 =head1 CHANGES
 
-=head2 Version 1.00
+=head2 Version 1.00 (dpkg 1.15.6)
 
 Mark the module as public.
 
-=head1 AUTHOR
-
-Raphaël Hertzog <hertzog@debian.org>.
-
 =cut
 
 1;
