@@ -22,6 +22,7 @@ our $VERSION = '0.01';
 
 use Cwd;
 use File::Basename;
+use File::Spec;
 use File::Temp qw(tempfile);
 
 use Dpkg::Gettext;
@@ -41,7 +42,6 @@ sub do_extract {
     my $sourcestyle = $self->{options}{sourcestyle};
     my $fields = $self->{fields};
 
-    my $dscdir = $self->{basedir};
     my $basename = $self->get_basename();
     my $basenamerev = $self->get_basename(1);
 
@@ -49,7 +49,7 @@ sub do_extract {
     my $comp_ext_regex = compression_get_file_extension_regex();
     foreach my $file ($self->get_files()) {
 	if ($file =~ /^\Q$basenamerev\E\.tar\.$comp_ext_regex$/) {
-            error(g_('multiple tarfiles in v1.0 source package')) if $tarfile;
+            error(g_('multiple tarfiles in native source package')) if $tarfile;
             $tarfile = $file;
 	} else {
 	    error(g_('unrecognized file for a native source package: %s'), $file);
@@ -65,7 +65,9 @@ sub do_extract {
     }
 
     info(g_('unpacking %s'), $tarfile);
-    my $tar = Dpkg::Source::Archive->new(filename => "$dscdir$tarfile");
+    my $tar = Dpkg::Source::Archive->new(
+        filename => File::Spec->catfile($self->{basedir}, $tarfile),
+    );
     $tar->extract($newdirectory);
 }
 
