@@ -1,4 +1,4 @@
-# Copyright © 2006-2009, 2012-2015 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2009, 2012-2020, 2022 Guillem Jover <guillem@debian.org>
 # Copyright © 2007-2010 Raphaël Hertzog <hertzog@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,12 +19,11 @@ package Dpkg::Substvars;
 use strict;
 use warnings;
 
-our $VERSION = '1.05';
-
-use POSIX qw(:errno_h);
+our $VERSION = '2.01';
 
 use Dpkg ();
 use Dpkg::Arch qw(get_host_arch);
+use Dpkg::Vendor qw(get_current_vendor);
 use Dpkg::Version;
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
@@ -41,8 +40,7 @@ Dpkg::Substvars - handle variable substitution in strings
 
 =head1 DESCRIPTION
 
-It provides some an object which is able to substitute variables in
-strings.
+It provides a class which is able to substitute variables in strings.
 
 =cut
 
@@ -50,6 +48,7 @@ use constant {
     SUBSTVAR_ATTR_USED => 1,
     SUBSTVAR_ATTR_AUTO => 2,
     SUBSTVAR_ATTR_AGED => 4,
+    SUBSTVAR_ATTR_OPT  => 8,
 };
 
 =head1 METHODS
@@ -171,25 +170,6 @@ sub mark_as_used {
     $self->{attr}{$key} |= SUBSTVAR_ATTR_USED;
 }
 
-=item $s->no_warn($key)
-
-Obsolete function, use mark_as_used() instead.
-
-=cut
-
-sub no_warn {
-    my ($self, $key) = @_;
-
-    warnings::warnif('deprecated',
-                     'obsolete no_warn() function, use mark_as_used() instead');
-
-    $self->mark_as_used($key);
-}
-
-=item $s->load($file)
-
-Add new substitutions read from $file.
-
 =item $s->parse($fh, $desc)
 
 Add new substitutions read from the filehandle. $desc is used to identify
@@ -206,19 +186,28 @@ sub parse {
 
     binmode($fh);
     while (<$fh>) {
+        my $attr;
+
 	next if m/^\s*\#/ || !m/\S/;
 	s/\s*\n$//;
-	if (! m/^(\w[-:0-9A-Za-z]*)\=(.*)$/) {
+	if (! m/^(\w[-:0-9A-Za-z]*)(\?)?\=(.*)$/) {
 	    error(g_('bad line in substvars file %s at line %d'),
 		  $varlistfile, $.);
 	}
-	$self->set($1, $2);
+        if (defined $2) {
+            $attr = (SUBSTVAR_ATTR_USED | SUBSTVAR_ATTR_OPT) if $2 eq '?';
+        }
+        $self->set($1, $3, $attr);
         $count++;
     }
 
     return $count
 }
 
+=item $s->load($file)
+
+Add new substitutions read from $file.
+
 =item $s->set_version_substvars($sourceversion, $binaryversion)
 
 Defines ${binary:Version}, ${source:Version} and
@@ -270,6 +259,44 @@ sub set_arch_substvars {
     $self->set('Arch', get_host_arch(), $attr);
 }
 
+=item $s->set_vendor_substvars()
+
+Defines vendor variables: ${vendor:Name} and ${vendor:Id}.
+
+These will never be warned about when unused.
+
+=cut
+
+sub set_vendor_substvars {
+    my ($self, $desc) = @_;
+
+    my $attr = SUBSTVAR_ATTR_USED | SUBSTVAR_ATTR_AUTO;
+
+    my $vendor = get_current_vendor();
+    $self->set('vendor:Name', $vendor, $attr);
+    $self->set('vendor:Id', lc $vendor, $attr);
+}
+
+=item $s->set_desc_substvars()
+
+Defines source description variables: ${source:Synopsis} and
+${source:Extended-Description}.
+
+These will never be warned about when unused.
+
+=cut
+
+sub set_desc_substvars {
+    my ($self, $desc) = @_;
+
+    my ($synopsis, $extended) = split /\n/, $desc, 2;
+
+    my $attr = SUBSTVAR_ATTR_USED | SUBSTVAR_ATTR_AUTO;
+
+    $self->set('source:Synopsis', $synopsis, $attr);
+    $self->set('source:Extended-Description', $extended, $attr);
+}
+
 =item $s->set_field_substvars($ctrl, $prefix)
 
 Defines field variables from a Dpkg::Control object, with each variable
@@ -324,7 +351,8 @@ sub substvars {
                       g_('obsolete substitution variable ${%s}'), $vn);
             }
         } else {
-            warning($opts{msg_prefix} . g_('unknown substitution variable ${%s}'),
+            warning($opts{msg_prefix} .
+                    g_('substitution variable ${%s} used, but is not defined'),
 	            $vn) unless $opts{no_warn};
             $v = $lhs . $rhs;
         }
@@ -342,13 +370,14 @@ sub warn_about_unused {
     my ($self, %opts) = @_;
     $opts{msg_prefix} //= $self->{msg_prefix};
 
-    foreach my $vn (keys %{$self->{vars}}) {
+    foreach my $vn (sort keys %{$self->{vars}}) {
         next if $self->{attr}{$vn} & SUBSTVAR_ATTR_USED;
         # Empty substitutions variables are ignored on the basis
         # that they are not required in the current situation
         # (example: debhelper's misc:Depends in many cases)
         next if $self->{vars}{$vn} eq '';
-        warning($opts{msg_prefix} . g_('unused substitution variable ${%s}'),
+        warning($opts{msg_prefix} .
+                g_('substitution variable ${%s} unused, but is defined'),
                 $vn);
     }
 }
@@ -385,20 +414,15 @@ sub filter {
     }
 }
 
-=item $s->save($file)
-
-Store all substitutions variables except the automatic ones in the
-indicated file.
-
 =item "$s"
 
 Return a string representation of all substitutions variables except the
 automatic ones.
 
-=item $str = $s->output($fh)
+=item $str = $s->output([$fh])
 
-Print all substitutions variables except the automatic ones in the
-filehandle and return the content written.
+Return all substitutions variables except the automatic ones. If $fh
+is passed print them into the filehandle.
 
 =cut
 
@@ -408,17 +432,37 @@ sub output {
     # Store all non-automatic substitutions only
     foreach my $vn (sort keys %{$self->{vars}}) {
 	next if $self->{attr}{$vn} & SUBSTVAR_ATTR_AUTO;
-	my $line = "$vn=" . $self->{vars}{$vn} . "\n";
+        my $op = $self->{attr}{$vn} & SUBSTVAR_ATTR_OPT ? '?=' : '=';
+        my $line = "$vn$op" . $self->{vars}{$vn} . "\n";
 	print { $fh } $line if defined $fh;
 	$str .= $line;
     }
     return $str;
 }
 
+=item $s->save($file)
+
+Store all substitutions variables except the automatic ones in the
+indicated file.
+
 =back
 
 =head1 CHANGES
 
+=head2 Version 2.01 (dpkg 1.21.8)
+
+New feature: Add support for optional substitution variables.
+
+=head2 Version 2.00 (dpkg 1.20.0)
+
+Remove method: $s->no_warn().
+
+New method: $s->set_vendor_substvars().
+
+=head2 Version 1.06 (dpkg 1.19.0)
+
+New method: $s->set_desc_substvars().
+
 =head2 Version 1.05 (dpkg 1.18.11)
 
 Obsolete substvar: Emit an error on Source-Version substvar usage.
