@@ -24,12 +24,11 @@ use warnings;
 
 our $VERSION = '0.01';
 
+use List::Util qw(any);
+
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
-use Dpkg::Path qw(find_command);
 use Dpkg::Control::Types;
-use Dpkg::BuildOptions;
-use Dpkg::Arch qw(debarch_eq get_host_arch);
 
 use parent qw(Dpkg::Vendor::Debian);
 
@@ -37,11 +36,11 @@ use parent qw(Dpkg::Vendor::Debian);
 
 =head1 NAME
 
-Dpkg::Vendor::Ubuntu - Ubuntu vendor object
+Dpkg::Vendor::Ubuntu - Ubuntu vendor class
 
 =head1 DESCRIPTION
 
-This vendor object customizes the behaviour of dpkg scripts for Ubuntu
+This vendor class customizes the behaviour of dpkg scripts for Ubuntu
 specific behavior and policies.
 
 =cut
@@ -57,8 +56,8 @@ sub run_hook {
         # https://wiki.ubuntu.com/DebianMaintainerField
         if (defined($fields->{'Version'}) and defined($fields->{'Maintainer'}) and
            $fields->{'Version'} =~ /ubuntu/) {
-           if ($fields->{'Maintainer'} !~ /ubuntu/i) {
-               if (length $ENV{DEBEMAIL} and $ENV{DEBEMAIL} =~ /\@ubuntu\.com/) {
+           if ($fields->{'Maintainer'} !~ /(?:ubuntu|canonical)/i) {
+               if (length $ENV{DEBEMAIL} and $ENV{DEBEMAIL} =~ /\@(?:ubuntu|canonical)\.com/) {
                    error(g_('Version number suggests Ubuntu changes, but Maintainer: does not have Ubuntu address'));
                } else {
                    warning(g_('Version number suggests Ubuntu changes, but Maintainer: does not have Ubuntu address'));
@@ -68,9 +67,6 @@ sub run_hook {
                warning(g_('Version number suggests Ubuntu changes, but there is no XSBC-Original-Maintainer field'));
            }
         }
-
-    } elsif ($hook eq 'keyrings') {
-        return $self->run_hook('package-keyrings', @params);
     } elsif ($hook eq 'package-keyrings') {
         return ($self->SUPER::run_hook($hook),
                 '/usr/share/keyrings/ubuntu-archive-keyring.gpg');
@@ -82,13 +78,15 @@ sub run_hook {
                 '/usr/share/keyrings/ubuntu-archive-removed-keys.gpg');
     } elsif ($hook eq 'register-custom-fields') {
         my @field_ops = $self->SUPER::run_hook($hook);
-        push @field_ops,
-            [ 'register', 'Launchpad-Bugs-Fixed',
-              CTRL_FILE_CHANGES | CTRL_CHANGELOG  ],
-            [ 'insert_after', CTRL_FILE_CHANGES, 'Closes', 'Launchpad-Bugs-Fixed' ],
-            [ 'insert_after', CTRL_CHANGELOG, 'Closes', 'Launchpad-Bugs-Fixed' ];
+        push @field_ops, [
+            'register', 'Launchpad-Bugs-Fixed',
+              CTRL_FILE_CHANGES | CTRL_CHANGELOG,
+        ], [
+            'insert_after', CTRL_FILE_CHANGES, 'Closes', 'Launchpad-Bugs-Fixed',
+        ], [
+            'insert_after', CTRL_CHANGELOG, 'Closes', 'Launchpad-Bugs-Fixed',
+        ];
         return @field_ops;
-
     } elsif ($hook eq 'post-process-changelog-entry') {
         my $fields = shift @params;
 
@@ -97,28 +95,44 @@ sub run_hook {
         if (scalar(@$bugs)) {
             $fields->{'Launchpad-Bugs-Fixed'} = join(' ', @$bugs);
         }
-
     } elsif ($hook eq 'update-buildflags') {
 	my $flags = shift @params;
-	my $build_opts = Dpkg::BuildOptions->new();
 
-	if (!$build_opts->has('noopt')) {
-	    if (debarch_eq(get_host_arch(), 'ppc64el')) {
-		for my $flag (qw(CFLAGS CXXFLAGS OBJCFLAGS OBJCXXFLAGS GCJFLAGS
-		                 FFLAGS FCFLAGS)) {
-		    $flags->set($flag, '-g -O3', 'vendor');
-		}
-	    }
-	}
+        # Run the Debian hook to add hardening flags
+        $self->SUPER::run_hook($hook, $flags);
+
 	# Per https://wiki.ubuntu.com/DistCompilerFlags
-	$flags->set('LDFLAGS', '-Wl,-Bsymbolic-functions', 'vendor');
-
-	# Run the Debian hook to add hardening flags
-	$self->SUPER::run_hook($hook, $flags);
+        $flags->prepend('LDFLAGS', '-Wl,-Bsymbolic-functions');
     } else {
         return $self->SUPER::run_hook($hook, @params);
     }
+}
 
+# Override Debian default features.
+sub init_build_features {
+    my ($self, $use_feature, $builtin_feature) = @_;
+
+    $self->SUPER::init_build_features($use_feature, $builtin_feature);
+
+    require Dpkg::Arch;
+    my $arch = Dpkg::Arch::get_host_arch();
+
+    if (any { $_ eq $arch } qw(amd64 arm64 ppc64el s390x)) {
+        $use_feature->{optimize}{lto} = 1;
+    }
+}
+
+sub set_build_features {
+    my ($self, $flags) = @_;
+
+    $self->SUPER::set_build_features($flags);
+
+    require Dpkg::Arch;
+    my $arch = Dpkg::Arch::get_host_arch();
+
+    if ($arch eq 'ppc64el' && $flags->get_option_value('optimize-level') != 0) {
+        $flags->set_option_value('optimize-level', 3);
+    }
 }
 
 =head1 PUBLIC FUNCTIONS
