@@ -42,11 +42,11 @@ use Dpkg::Arch qw(
     get_host_arch
     debarch_eq debarch_to_gnutriplet
 );
-use Dpkg::Build::Types;
-use Dpkg::Build::Info qw(get_build_env_allowed);
+use Dpkg::BuildTypes;
 use Dpkg::BuildOptions;
 use Dpkg::BuildFlags;
 use Dpkg::BuildProfiles qw(get_build_profiles);
+use Dpkg::BuildInfo qw(get_build_env_allowed);
 use Dpkg::Control::Info;
 use Dpkg::Control::Fields;
 use Dpkg::Control;
@@ -285,9 +285,13 @@ sub is_cross_executable {
         nocheck => 1,
     );
     if ($?) {
-       print { *STDOUT } $stdout;
-       print { *STDERR } $stderr;
-       subprocerr("$CC -w -x c -");
+        print { *STDOUT } $stdout;
+        print { *STDERR } $stderr;
+        eval {
+            subprocerr("$CC -w -x c -");
+        };
+        warning($@);
+        return;
     }
     close $tmpfh;
     spawn(
