@@ -160,6 +160,10 @@ while (@options) {
 	usageerr(g_('%s is not a compression level'), $comp_level)
 	    unless compression_is_valid_level($comp_level);
 	compression_set_default_level($comp_level);
+    } elsif (m/^--threads-max=(.*)$/) {
+        my $threads = $1;
+        $options{comp_threads} = $threads;
+        compression_set_threads($threads);
     } elsif (m/^-c(.*)$/) {
         $controlfile = $1;
     } elsif (m/^-l(.*)$/) {
@@ -226,9 +230,7 @@ unless (defined($options{opmode})) {
 }
 
 if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
-
     $options{ARGV} = \@ARGV;
-
     $options{changelog_file} ||= "$dir/debian/changelog";
     $controlfile ||= "$dir/debian/control";
 
@@ -298,7 +300,7 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 	my $sect = $pkg->{'Section'} || $src_sect;
 	my $prio = $pkg->{'Priority'} || $src_prio;
 	my $type = $pkg->{'Package-Type'} ||
-	        $pkg->get_custom_field('Package-Type') || 'deb';
+            $pkg->get_custom_field('Package-Type') || 'deb';
         my $arch = $pkg->{'Architecture'};
         my $profile = $pkg->{'Build-Profiles'};
 
@@ -441,9 +443,7 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 		       override => \%override,
 		       substvars => $substvars);
     exit(0);
-
 } elsif ($options{opmode} eq 'extract') {
-
     # Check command line
     unless (scalar(@ARGV)) {
         usageerr(g_('--%s needs at least one argument, the .dsc'),
@@ -654,6 +654,8 @@ sub usage {
     get_format_help()
     . "\n" . g_(
 'General options:
+      --threads-max=<threads>
+                           use at most <threads> with compressor.
   -q                       quiet mode.
   -?, --help               show this help message.
       --version            show the version.')
